
/*  
 * Copyright (c) 2007, Sun Microsystems, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the name of Sun Microsystems nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *  Note:  In order to comply with the binary form redistribution 
 *         requirement in the above license, the licensee may include 
 *         a URL reference to a copy of the required copyright notice, 
 *         the list of conditions and the disclaimer in a human readable 
 *         file with the binary form of the code that is subject to the
 *         above license.  For example, such file could be put on a 
 *         Blu-ray disc containing the binary form of the code or could 
 *         be put in a JAR file that is broadcast via a digital television 
 *         broadcast medium.  In any event, you must include in any end 
 *         user licenses governing any code that includes the code subject 
 *         to the above license (in source and/or binary form) a disclaimer 
 *         that is at least as protective of Sun as the disclaimers in the 
 *         above license.
 * 
 *         A copy of the required copyright notice, the list of conditions and
 *         the disclaimer will be maintained at 
 *         https://hdcookbook.dev.java.net/misc/license.html .
 *         Thus, licensees may comply with the binary form redistribution
 *         requirement with a text file that contains the following text:
 * 
 *             A copy of the license(s) governing this code is located
 *             at https://hdcookbook.dev.java.net/misc/license.html
 */

package net.java.bd.tools.bdjo;

/**
 * TableOfAccessiblePlayLists - section 10.2.4
 * @author A. Sundararajan
 */
public class TableOfAccessiblePlayLists {
    // TableOfAccessiblePlayLists syntax - section 10.2.4.2
    // number_of_acc_PlayLists is specified in 11 bits
    public static final int NUM_PLAYLISTS_BITS = 11;
    public static final int MAX_PLAYLISTS = (1 << NUM_PLAYLISTS_BITS);
    
    // access_to_all_flag - section 10.2.4.3
    private boolean accessToAllFlag;
    // autostart_first_PlayList_flag - section 10.2.4.3
    private boolean autostartFirstPlayListFlag;
    // PlayList_file_name in for-loop - section 10.2.4.3
    private String[] playListFileNames;

    public boolean isAccessToAllFlag() {
        return accessToAllFlag;
    }

    public void setAccessToAllFlag(boolean accessToAllFlag) {
        this.accessToAllFlag = accessToAllFlag;
    }

    public boolean isAutostartFirstPlayListFlag() {
        return autostartFirstPlayListFlag;
    }

    public void setAutostartFirstPlayListFlag(boolean autostartFirstPlayListFlag) {
        this.autostartFirstPlayListFlag = autostartFirstPlayListFlag;
    }

    public String[] getPlayListFileNames() {
        return playListFileNames;
    }

    public void setPlayListFileNames(String[] playListFileNames) {
        if (playListFileNames != null) {           
            if (playListFileNames.length > MAX_PLAYLISTS) {
                throw new IllegalArgumentException("overflow play lists : " +
                    playListFileNames.length);
            }
            for (String pl : playListFileNames) {
                if (pl != null) {
                    BDJO.checkFileName(pl);
                }
            }
        }
        this.playListFileNames = playListFileNames;
    }
}
